package com.example.demo;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.service.OrderService;
import com.example.demo.products.model.ProductEntity;
import com.example.demo.products.service.ProductService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
	private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

	private final TypeService typeService;
	private final ProductService productService;
	private final UserService userService;
	private final OrderService orderService;

	public DemoApplication(TypeService typeService, ProductService productService, UserService userService,
			OrderService orderService) {
		this.typeService = typeService;
		this.productService = productService;
		this.userService = userService;
		this.orderService = orderService;
	}

	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	@Override
	public void run(String... args) throws Exception {
		if (args.length > 0 && Objects.equals("--populate", args[0])) {
			log.info("Create default types values");
			final var type1 = typeService.create(new TypeEntity(null, "Пицца"));
			final var type2 = typeService.create(new TypeEntity(null, "Напиток"));
			final var type3 = typeService.create(new TypeEntity(null, "Закуска"));

			log.info("Create default products values");
			productService.create(new ProductEntity(null, "Маргарита", type1, 499.00));
			productService.create(new ProductEntity(null, "Эль Дьябло", type1, 699.00));
			productService.create(new ProductEntity(null, "Гавайская", type1, 399.00));
			productService.create(new ProductEntity(null, "Лимонад", type2, 99.00));
			productService.create(new ProductEntity(null, "Сок", type2, 99.00));
			productService.create(new ProductEntity(null, "Чай", type2, 49.00));
			productService.create(new ProductEntity(null, "Картошка фри", type3, 199.00));
			productService.create(new ProductEntity(null, "Нагетсы", type3, 199.00));

			log.info("Create default users values");
			userService.create(new UserEntity(null, "Alex", "Kryukov", "akryu@mail.ru", "password123"));
			userService.create(new UserEntity(null, "Oleg", "Zyngin", "@mail.ru", "password"));

			log.info("Create default orders values");
			@SuppressWarnings({ "rawtypes", "unchecked" })
			List<OrderLineEntity> lines = new ArrayList();
			lines.add(new OrderLineEntity(null, null, 3, null));
			final var user1 = userService.create(new UserEntity(null, "Alex", "Kryukov", "akryu@mail.ru", "password"));
			orderService.create(new OrderEntity(null, user1, lines));
		}
	}
}
