package com.example.demo.order_lines.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import com.example.demo.core.configuration.Constants;
import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.order_lines.service.OrderLineService;
import com.example.demo.products.model.ProductEntity;
import com.example.demo.products.service.ProductService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/orderLine")
public class OrderLineController {
    private final OrderLineService orderLineService;
    private final ProductService productService;
    private final ModelMapper modelMapper;

    public OrderLineController(OrderLineService orderLineService,
            ProductService productService, ModelMapper modelMapper) {
        this.orderLineService = orderLineService;
        this.productService = productService;
        this.modelMapper = modelMapper;
    }

    private OrderLineDto toDto(OrderLineEntity orderLineEntity) {
        return modelMapper.map(orderLineEntity, OrderLineDto.class);
    }

    private OrderLineEntity toEntity(OrderLineDto dto) {
        // Получаем заказ и товар из соответствующих сервисов по их идентификаторам из
        ProductEntity product = productService.get(dto.getProductId());

        // Создаем новый экземпляр OrderLineEntity с помощью модельного маппера
        OrderLineEntity entity = modelMapper.map(dto, OrderLineEntity.class);

        // Устанавливаем полученные заказ и товар в OrderLineEntity
        entity.setProduct(product);

        return entity;
    }

    @GetMapping
    public List<OrderLineDto> getAll(@RequestParam(name = "orderId", defaultValue = "0") Long orderId) {
        return orderLineService.getAll(orderId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public OrderLineDto get(@PathVariable(name = "id") Long id) {
        return toDto(orderLineService.get(id));
    }

    @PostMapping
    public OrderLineDto create(@RequestBody @Valid OrderLineDto dto) {
        // Получаем информацию о продукте по его ID
        ProductEntity product = productService.get(dto.getProductId());

        // Рассчитываем общую сумму строки заказа
        double totalPrice = product.getPrice() * dto.getCount();

        // Создаем новую сущность строки заказа
        OrderLineEntity orderLineEntity = toEntity(dto);

        // Устанавливаем общую сумму в сущность строки заказа
        orderLineEntity.setTotalPrice(totalPrice);

        // Создаем строку заказа
        OrderLineEntity createdOrderLine = orderLineService.create(orderLineEntity);

        // Преобразуем созданную строку заказа в DTO
        OrderLineDto createdOrderLineDto = toDto(createdOrderLine);

        // Возвращаем созданную строку заказа в виде DTO с заполненным полем totalPrice
        return createdOrderLineDto;
    }

    @PutMapping("/{id}")
    public OrderLineDto update(@PathVariable(name = "id") Long id, @RequestBody OrderLineDto dto) {
        return toDto(orderLineService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public OrderLineDto delete(@PathVariable(name = "id") Long id) {
        return toDto(orderLineService.delete(id));
    }
}
