package com.example.demo.orders.api;

import java.sql.Date;
import java.util.ArrayList;
import java.util.List;
import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import com.example.demo.core.configuration.Constants;
import com.example.demo.order_lines.api.OrderLineDto;
import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.order_lines.service.OrderLineService;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.service.OrderService;
import com.example.demo.users.service.UserService;
import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/order")
public class OrderController {
    private final OrderService orderService;
    private final UserService userService;
    private final ModelMapper modelMapper;
    private final OrderLineService orderLineService;

    public OrderController(OrderService orderService, UserService userService, ModelMapper modelMapper,
            OrderLineService orderLineService) {
        this.orderService = orderService;
        this.userService = userService;
        this.modelMapper = modelMapper;
        this.orderLineService = orderLineService;
    }

    private OrderDto toDto(OrderEntity entity) {
        var dto = new OrderDto();
        dto.setId(entity.getId());
        dto.setUserId(entity.getUser().getId());

        // Преобразование каждой сущности OrderLineEntity в OrderLineDto и добавление их
        // в список lines
        for (OrderLineEntity orderLineEntity : entity.getLines()) {
            OrderLineDto orderLineDto = new OrderLineDto();
            orderLineDto.setId(orderLineEntity.getId());
            orderLineDto.setProductId(orderLineEntity.getProduct().getId());
            orderLineDto.setCount(orderLineEntity.getCount());
            orderLineDto.setTotalPriceLine(orderLineEntity.getTotalPrice());
            dto.addOrderLine(orderLineDto);
        }

        dto.setDate(new Date(System.currentTimeMillis()));
        return dto;
    }

    public OrderEntity toEntity(OrderDto dto) {
        final OrderEntity entity = modelMapper.map(dto, OrderEntity.class);
        if (dto.getUserId() != null) {
            entity.setUser(userService.get(dto.getUserId()));
        }
        // устанавливаем связь между заказом и строками заказа
        entity.getLines().clear();
        for (OrderLineDto lineDto : dto.getLines()) {
            OrderLineEntity orderLineEntity = modelMapper.map(lineDto, OrderLineEntity.class);
            entity.addOrderLine(orderLineEntity);
        }
        return entity;
    }

    @GetMapping
    public List<OrderDto> getAll(@RequestParam(name = "userId", defaultValue = "0") Long userId,
            @RequestParam(name = "lines", defaultValue = "") List<Long> lines) {
        return orderService.getAll(userId, lines).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public OrderDto get(@PathVariable(name = "id") Long id) {
        return toDto(orderService.get(id));
    }

    @PostMapping
    public OrderDto create(@RequestBody @Valid OrderDto dto) {
        OrderEntity orderEntity = toEntity(dto);
        return toDto(orderService.create(orderEntity));
    }

    @PutMapping("/{id}")
    public OrderDto update(@PathVariable(name = "id") Long id, @RequestBody OrderDto dto) {
        return toDto(orderService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public OrderDto delete(@PathVariable(name = "id") Long id) {
        return toDto(orderService.delete(id));
    }
}
