package com.example.demo.orders.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.order_lines.service.OrderLineService;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.repository.OrderRepository;

@Service
public class OrderService {
    private final OrderRepository repository;
    private final OrderLineService orderLineService;

    public OrderService(OrderRepository repository, OrderLineService orderLineService) {
        this.repository = repository;
        this.orderLineService = orderLineService;
    }

    public List<OrderEntity> getAll(Long userId, List<Long> lines) {

        var linesEnt = new ArrayList<>();
        for (var lineid : lines) {
            linesEnt.add(orderLineService.get(lineid));
        }

        if (!Objects.equals(userId, 0L) && !Objects.equals(linesEnt.size(), 0)) {
            return repository.getAll().stream()
                    .filter(line -> line.getUser().getId().equals(
                            userId) && line.getLines().containsAll(linesEnt))
                    .toList();
        }
        if (Objects.equals(userId, 0L) && !Objects.equals(linesEnt.size(), 0)) {
            return repository.getAll().stream().filter(line -> line.getLines().containsAll(linesEnt)).toList();
        }
        if (!Objects.equals(userId, 0L) && Objects.equals(linesEnt.size(), 0)) {
            return repository.getAll().stream().filter(line -> line.getUser().getId().equals(userId)).toList();
        }
        return repository.getAll();
    }

    public OrderEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public OrderEntity create(OrderEntity entity) {
        return repository.create(entity);
    }

    public OrderEntity update(Long id, OrderEntity entity) {
        final OrderEntity existEntity = get(id);
        existEntity.setUser(entity.getUser());
        existEntity.getLines().clear(); // Очищаем существующие строки заказа
        var lines = entity.getLines();
        for (var line : lines) {
            existEntity.setLines(line);
        }
        return repository.update(existEntity);
    }

    public OrderEntity delete(Long id) {
        final OrderEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
