package com.example.demo.products.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import com.example.demo.core.configuration.Constants;
import com.example.demo.products.model.ProductEntity;
import com.example.demo.products.service.ProductService;
import com.example.demo.types.service.TypeService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/product")
public class ProductController {
    private final ProductService productService;
    private final TypeService typeService;
    private final ModelMapper modelMapper;

    public ProductController(ProductService productService, TypeService typeService, ModelMapper modelMapper) {
        this.productService = productService;
        this.typeService = typeService;
        this.modelMapper = modelMapper;
    }

    private ProductDto toDto(ProductEntity entity) {
        return modelMapper.map(entity, ProductDto.class);
    }

    private ProductEntity toEntity(ProductDto dto) {
        final ProductEntity entity = modelMapper.map(dto, ProductEntity.class);
        entity.setType(typeService.get(dto.getTypeId()));
        return entity;
    }

    @GetMapping
    public List<ProductDto> getAll(@RequestParam(name = "typeId", defaultValue = "0") Long typeId) {
        return productService.getAll(typeId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public ProductDto get(@PathVariable(name = "id") Long id) {
        return toDto(productService.get(id));
    }

    @PostMapping
    public ProductDto create(@RequestBody @Valid ProductDto dto) {
        return toDto(productService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public ProductDto update(@PathVariable(name = "id") Long id, @RequestBody ProductDto dto) {
        return toDto(productService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public ProductDto delete(@PathVariable(name = "id") Long id) {
        return toDto(productService.delete(id));
    }
}
