package com.example.demo.products.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.products.model.ProductEntity;
import com.example.demo.products.repository.ProductRepository;

@Service
public class ProductService {
    private final ProductRepository repository;

    public ProductService(ProductRepository repository) {
        this.repository = repository;
    }

    public List<ProductEntity> getAll(Long typeId) {
        if (Objects.equals(typeId, 0L)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(order -> order.getType().getId().equals(typeId))
                .toList();
    }

    public ProductEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public ProductEntity create(ProductEntity entity) {
        return repository.create(entity);
    }

    public ProductEntity update(Long id, ProductEntity entity) {
        final ProductEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        existsEntity.setType(entity.getType());
        existsEntity.setPrice(entity.getPrice());
        return repository.update(existsEntity);
    }

    public ProductEntity delete(Long id) {
        final ProductEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
