package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.order_lines.service.OrderLineService;

@SpringBootTest
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
class OrderLineServiceTests {
    @Autowired
    private OrderLineService orderLineService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> orderLineService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        // Создаем тестовую сущность OrderLineEntity
        OrderLineEntity testOrderLine = new OrderLineEntity(null, null, 5, null);
        // Вызываем метод create() и сохраняем созданную сущность
        OrderLineEntity createdOrderLine = orderLineService.create(testOrderLine);
        // Проверяем, что метод create() вернул не null
        Assertions.assertNotNull(createdOrderLine);
        // Проверяем, что созданная сущность имеет назначенный ID
        Assertions.assertNotNull(createdOrderLine.getId());
        // Проверяем, что созданная сущность соответствует той, которую мы передали
        Assertions.assertEquals(testOrderLine.getCount(), createdOrderLine.getCount());
    }

    @Test
    @Order(2)
    void updateTest() {
        // Получаем сущность OrderLineEntity для обновления
        OrderLineEntity existingOrderLine = orderLineService.get(1L);
        // Создаем новое количество для обновления
        int newCount = 10;
        // Устанавливаем новое количество в сущность OrderLineEntity
        existingOrderLine.setCount(newCount);
        // Вызываем метод update() и сохраняем обновленную сущность
        OrderLineEntity updatedOrderLine = orderLineService.update(1L, existingOrderLine);
        // Проверяем, что метод update() вернул не null
        Assertions.assertNotNull(updatedOrderLine);
        // Проверяем, что обновленная сущность имеет то же количество, что и заданное
        Assertions.assertEquals(newCount, updatedOrderLine.getCount());
    }

    @Test
    @Order(3)
    void deleteTest() {
        // Удаляем сущность OrderLineEntity по ее ID
        OrderLineEntity deletedOrderLine = orderLineService.delete(1L);
        // Проверяем, что метод delete() вернул не null
        Assertions.assertNotNull(deletedOrderLine);
        // Проверяем, что удаленная сущность имеет тот же ID, что и удаленная
        Assertions.assertEquals(1L, deletedOrderLine.getId());
        // Проверяем, что сущность больше не существует в репозитории
        Assertions.assertThrows(NotFoundException.class, () -> orderLineService.get(1L));
    }
}
