package com.example.demo;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.service.OrderService;

@SpringBootTest
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
class OrderServiceTests {
    @Autowired
    private OrderService orderService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> orderService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        orderService.create(new OrderEntity(null, null, null));
        @SuppressWarnings({ "rawtypes", "unchecked" })
        List<OrderLineEntity> lines = new ArrayList();
        lines.add(new OrderLineEntity(null, null, 4, null));
        lines.add(new OrderLineEntity(null, null, 5, null));
        lines.add(new OrderLineEntity(null, null, 6, null));
        lines.add(new OrderLineEntity(null, null, 7, null));
        // Создаем тестовую сущность OrderEntity
        OrderEntity testOrder = new OrderEntity(null, null, lines);
        // Вызываем метод create() и сохраняем созданную сущность
        OrderEntity createdOrder = orderService.create(testOrder);
        // Проверяем, что метод create() вернул не null
        Assertions.assertNotNull(createdOrder);
        // Проверяем, что созданная сущность имеет назначенный ID
        Assertions.assertNotNull(createdOrder.getId());
    }

    @Test
    @Order(2)
    void updateTest() {
        // Получаем сущность OrderEntity для обновления
        OrderEntity existingOrder = orderService.get(1L);
        // Вызываем метод update() и сохраняем обновленную сущность
        OrderEntity updatedOrder = orderService.update(1L, existingOrder);
        // Проверяем, что метод update() вернул не null
        Assertions.assertNotNull(updatedOrder);
    }

    @Test
    @Order(3)
    void deleteTest() {
        // Удаляем сущность OrderEntity по ее ID
        OrderEntity deletedOrder = orderService.delete(1L);
        // Проверяем, что метод delete() вернул не null
        Assertions.assertNotNull(deletedOrder);
        // Проверяем, что удаленная сущность имеет тот же ID, что и удаленная
        Assertions.assertEquals(1L, deletedOrder.getId());
        // Проверяем, что сущность больше не существует в репозитории
        Assertions.assertThrows(NotFoundException.class, () -> orderService.get(1L));
    }
}
