package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.products.model.ProductEntity;
import com.example.demo.products.service.ProductService;
import com.example.demo.types.model.TypeEntity;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class ProductServiceTests {
    @Autowired
    private ProductService productService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> productService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {

        final var type1 = new TypeEntity(null, "Пицца");
        productService.create(new ProductEntity(null, "Mocarela", type1, 20.00));
        productService.create(new ProductEntity(null, "El'Diablo", type1, 20.00));
        productService.create(new ProductEntity(null, "Маргарита", type1, 499.00));
        productService.create(new ProductEntity(null, "Эль Дьябло", type1, 699.00));
        productService.create(new ProductEntity(null, "Гавайская", type1, 399.00));
        Assertions.assertEquals(5, productService.getAll(0L).size());
    }

    @Test
    @Order(2)
    void updateTest() {
        final ProductEntity newProduct = new ProductEntity(null, "El'Diablo", new TypeEntity(null, "Пицца"), 20.00);
        final ProductEntity updProduct = productService.update(1L, newProduct);
        Assertions.assertEquals(5, productService.getAll(0L).size());
        Assertions.assertEquals(updProduct, productService.get(1L));
        Assertions.assertEquals(newProduct.getName(), updProduct.getName());
        Assertions.assertEquals(newProduct.getPrice(), updProduct.getPrice());
    }

    @Test
    @Order(3)
    void deleteTest() {
        productService.delete(2L);
        Assertions.assertEquals(4L, productService.getAll(0L).size());
    }
}